import { ElementRef, EventEmitter, OnInit, AfterViewInit, QueryList } from '@angular/core';
import { ACService } from './slarn-autocomplete.service';
import { ControlValueAccessor } from '@angular/forms';
export declare class SlarnAutocompleteComponent implements OnInit, AfterViewInit, ControlValueAccessor {
    private _service;
    private _templateVariables;
    private _selectedId;
    _selectedItem: any | Array<SelectedItem>;
    /**
     * list contains code of keys that will trigger the search function
     * and the keys that represent navigation action
     * @link https://www.cambiaresearch.com/articles/15/javascript-char-codes-key-codes
     */
    private _keys;
    private propagateChange;
    private typingTimer;
    private doneTypingInterval;
    private _selectedIndexFromNavigation;
    displaySuggestions: boolean;
    loadingData: boolean;
    filteredItems: Array<any>;
    groups: Array<string>;
    filteredGroupedItems: any;
    translator: any;
    autocompleteInput: ElementRef;
    displayAllBtn: ElementRef;
    container: ElementRef;
    spanX: ElementRef;
    suggestions: QueryList<SlarnAutocompleteSuggestionComponent>;
    configuration: any;
    onItemSelected: EventEmitter<any>;
    constructor(_service: ACService);
    /**
     * Return the selected key(s)
     */
    selectedId: number | string | Array<number | string>;
    /**
     * Filter given value and make sure that the autocomplete gets the correct data
     * to work as expected
     * @param value
     */
    private filterSelectedValue(value);
    /**
     * Check array contain one or many objects
     * @param value
     */
    private arrayHasObject(value);
    /**
     * Return the selected item(s)
     */
    readonly selectedItem: any;
    ngOnInit(): void;
    ngAfterViewInit(): void;
    /**
     * Listener to check if user clicked outside of the autocomplete
     * If he did then we should hide the suggestion list
     * So we make sure that the autocomplete acts like any other list
     */
    checkIfClickedInside: (event: Event) => void;
    /**
     * Clear autocomplete selection
     */
    clearAutocomplete(): void;
    /**
     * Hide or display suggestions list
     */
    toggleDisplaySuggestions(): void;
    /**
     * Init default configuration
     */
    private initConfiguration();
    private prepareUsedLanguage();
    /**
     * Search and select item by key value
     */
    private searchAndSelectItemFromKey();
    /**
     * Search and select item from data by the value of the key
     * @param data: any[]
     */
    private selectItemFromData(data);
    /**
     * Extract available keys from template
     * will be used to dislay data in the suggestions panel
     */
    private extractTemplateVariables();
    /**
     * fired each time a user press a key
     * @param $event
     */
    onKeyup($event: any): void;
    private navigate(key);
    /**
     * Get the right selected item when pressing enter key
     * @param i
     * @returns right item
     */
    private getItemFromGroup(i);
    private enableSelectionForSelectedSuggestion(index);
    /**
     *
     */
    private clearAllSelections();
    /**
     * Delete item from selected list and dispatch changes
     * @param indexInSelectedItems
     * @param si
     */
    deleteFromSelectedItems(indexInSelectedItems: number, si: SelectedItem): void;
    /**
     * After key down clear used timer to calculate
     * when user finished typing
     */
    onKeyDown($event: any): void;
    /**
     * Check if this an alphabet or number key
     * @param $event keyup event
     * @return fireKeySearch
     */
    private fireSearchKey($event);
    /**
     * Check if pressed key is a navigation key
     * @param $event
     * @returns
     */
    private navigationKey($event);
    /**
     * If it's a local configuration then we will search inside the configuration.data object
     * @param word word to search
     * @param data filtered data
     */
    private searchLocally(word, data);
    /**
     * Check if given item exist in _selectedItem array ot not
     * @param item
     */
    private existInSelectedItems(item);
    /**
     * If it's a remote configuration then we get the word and add it to the url
     * before sending the request to the api
     * @param word word to search
     * @param url api url
     * @param selectItemAfterSearch
     */
    private searchRemotely(word, url, selectItemAfterSearch?);
    /**
     * If grouping is requested by the user then we need to prepare it
     */
    private buildGroupsIfNeeded();
    /**
     * Pares array of items and return grouped object by the given field
     * @param list
     * @param keyGetter
     * @returns Grouped object
     */
    private group(list, keyGetter);
    /**
     * Build view with data based on the given template
     * @param object
     * @return string built view
     */
    buildSuggestionView(object: any): string;
    /**
     * Build the view of the group based on giving template
     * @param group
     * @returns
     */
    buildGroupView(group: string): string;
    /**
     * Extract the correct value from the multidimensional object
     * @param keysString: string with keys separated by dots
     * @param object
     * @returns correct value
     */
    private extractValue(keysString, object);
    /**
     * Triggered after a user select a suggestion
     * @param item selected item from the list
     */
    performSelection(item: any): void;
    /**
     * Dispatch data to external components
     */
    private dispatchData();
    /**
     * Extract items from _selectedItems
     */
    private extractSelectedItems();
    registerOnChange(fn: any): void;
    registerOnTouched(fn: any): void;
    writeValue(value: any): void;
    /**
     * Open suggestions list
     */
    openSuggestions(): void;
    /**
     * Close suggestions list
     */
    closeSuggestions(): void;
    /**
     * Add new item to data
     * If there is another item with the same "key" value then the it will not be added
     * @param item
     * @param selectIt boolean: select the item after adding if true (false by default)
     */
    appendItem(item: any, selectIt: boolean): void;
    /**
     * Find if item already exist in data
     * @param data
     * @param item
     * @returns
     */
    private findItem(data, item);
}
/**
 * Suggestion component for Autocomplete
 */
export declare class SlarnAutocompleteSuggestionComponent implements OnInit {
    item: any;
    onSuggestionClicked: EventEmitter<any>;
    focusSuggestion: boolean;
    constructor();
    ngOnInit(): void;
    selectItem(): void;
}
/**
 * AutoComplete configuration
 */
export interface Configuration {
    /**
     * key will be stored in the input
     * must be the unique value of the object (ex: id)
     * will be used to search item and when sending form
     */
    key: string;
    /**
     * value will be displayed to the user in the input
     */
    value: string;
    /**
     * template that will be displayed in the suggestions list
     * if this attribute is not defined then the autocomplete will
     * use the default template and display only the value
     */
    template?: string;
    /**
     * The language that will be used to display default texts and errors
     */
    language?: string;
    /**
     * RTL Support (false by default)
     */
    rtl?: boolean;
    /**
     * name will be giving to the input
     */
    name?: string;
    /**
     * allow multiple selection (default: false)
     */
    multiple?: boolean;
    /**
     * allow to group items by a specific field
     */
    group?: Group;
}
/**
 * Remote configuration used when you want to work remotely with an api
 */
export interface ACRemoteConfiguration extends Configuration {
    /**
     * URL to the API
     */
    url: string;
    /**
     * number of characters needed before calling api
     */
    minCharacters?: number;
    /**
     * text or html will be rendered when loading data remotely
     */
    loadingView?: string;
}
/**
 * Local configuration: used if you want to pass data explicitly to the AutoComplete
 */
export interface ACLocalConfiguration extends Configuration {
    data: Array<any>;
}
/**
 * Group items
 */
export interface Group {
    /**
     * Arrow function to specify the group field
     * Used arrow function instead of a simple field cuz the group field can be a field of another complex object
     */
    field: any;
    /**
     * Block that will be rendered for the group
     */
    template?: string;
}
/**
 * We will use this object to store selected items
 */
export interface SelectedItem {
    elem: any;
    indexInFilteredItems: number;
}
